      subroutine davhes(grad,dgrad,normm,ic,happ,lam,evec,
     * avib,vvec,olam,iti,lunit,dconv,isel,nsel,csel,oldnorm,rvec,
     * ovvec,vtemp,nbm,onsel,aphs,hguess,
     * trm,notrro,oisel,mxres,ibm,nbmo,ortmod,nvara)

*==================================================================
* (Jacobi-)Davidson-diagonalization of the Hessian:
* calculate vibrational normal modes and corresponding frequency
* for selected modes according to a Davidson-like diagonalization
* scheme.
*
*------------------------------------------------------------------
* CAUTION: array normm: 
* ---------------------
* on input : mass-weighted basis vectors 
* on output: cartesian basis vectors; not mass-weighted
*------------------------------------------------------------------
*
* ortmod   : set of nbmo modes in orthogonal subspace to searchspace
*            (if nbmo > 0, explicit orthogonalization to these modes is done)
* lam      : vector of eigenvalues of the small Davidson Matrix in this iteration
* olam     : vector of eigenvalues of the small Davidson Matrix in the last iteration
* nbm      : No. of basis modes in this iteration
* onbm     : No. of basis modes in the last iteration
* nsel     : on input: No. of vectors selected for optimization in last iteration
*                      = nbm - onbm (there is one new basis vector for each vector
*                      to be optimized)
*            on output: No. of vectors selected for optimization in this iteration
* onsel    : original nsel, i.e., nsel in the first iteration
* isel(onsel) : isel(i) = Current no. of the basis vector which was obtained 
*                         by optimization of basis vector i (of the initial guess)
* oisel(onsel): oisel(i) = isel(i-1)
* csel(onsel) : csel(i) = .true., if vector isel(i) of the current iteration is
*                         converged
* ibm(onsel) : ibm(i)   = normal mode(nbm+i) corresponds to vector isel(ibm(i)) 
*                         in this iteration
*
* => absolute value of last basisvector could serve as convergence criterion,
*    but in order to check convergence of basis vector i, it is necessary to
*    consider the coefficient for basis mode(nbm+i)
* - Problem: what shall be done if the convergence criteria were fulfilled in
*   a former iteration ?
*
* written by J. Neugebauer, Uni Erlangen, 2002
* Jacobi-Davidson by J. Neugebauer, 2004
* slightly modified by C. Herrmann, ETHZ, 2006
*===================================================================

#include "defs.h"
#include "param.h"

      real*8, allocatable :: work(:),work2(:)
      integer, allocatable :: iwork(:)
      integer ic(natoms),iti,i,j,disptype,lwork,ilaenv,nsel
      integer info,k,isel(mxnsel),lunit,nbm,nsel2,ibm(mxnsel)
      integer onbm,onsel,count,l,oisel(mxnsel),jj
      integer icount,nbmo,nvara
      real*8 grad(nvar,naus,maxnbm),normm(nvar,maxnbm),cstepq
      real*8 dgrad(nvar,maxnbm),haij,dotprod,qfc2freq,find_mass
      real*8 happ(maxnbm,maxnbm),lam(maxnbm),evec(maxnbm,maxnbm)
      real*8 prec,thres,norm,avib(maxnbm),vvec(nvar,maxnbm),dvib
      real*8 olam(maxnbm),oldnorm(mxnsel),rvec(nvar,maxnbm),
     *       ovvec(nvar,maxnbm),scr,mxres(mxnsel),trm(nvar,6),
     *       hguess(nvara,nvara),ovl,epsilon,ortmod(nvar,nbmo)
      real*8 vtemp(nvar),kdelta
      parameter(thres=1.0d-4)
      logical dconv,para,csel(mxnsel),aphs,dcsel
      logical notrro,checkovl
      character*80 text
      external dotprod,qfc2freq,find_mass,kdelta,checkovl

*---------------
* initialization
*---------------
      do i=1,mxnsel
        ibm(i)=0
      end do

*----------------------------------------
* no. of basis vectors in last iteration
*----------------------------------------
      onbm=nbm-nsel+1
      disptype=1
      call rkeyint('akira_control','$disptype',disptype)

      write(ftape,*)
      write(ftape,'(2x,a)') '::::::::::::::::::::::::::::::::::::::::'
      write(ftape,'(2x,a,i5)') '    Results for iteration No.:',iti
      write(ftape,'(2x,a,i5)') '    No. of basis vectors     :',nbm
      write(ftape,'(2x,a)') '::::::::::::::::::::::::::::::::::::::::'
      write(ftape,*)

*================================================================
* 2) calculate vector s_i ("dgrad") for each new basis vector
*================================================================
      do j=onbm,nbm

*----------------------------------------------------------------
* determine step size
*----------------------------------------------------------------
        if(disptype.eq.1) then
          call getcstepq(normm(1,j),cstep,cstepq,ic,natoms)
        else
          stop 'disptype not available'
        end if
        write(ftape,'(a,i4,a,f10.6)')
     *  '  step size for mode no. ',j,' : ',cstepq

*----------------------------------------------------------------
* do numerical differentiation (this yields the vectors s_i=H*b_i)
*----------------------------------------------------------------
        call d_grad(grad,dgrad(1,j),naus,nvar,cstepq,j,maxnbm)

*----------------------------------------------------------------
* transform vectors H*b_i to mass-weighted coordinates
* (note: mass-weighting for differentiation with respect to 
* basis mode is already done)
*----------------------------------------------------------------
        do i=1,nvar
          dgrad(i,j)=dgrad(i,j)/sqrt(find_mass(ic((i+2)/3)))
        end do
        if(printall) then
          write(text, '(a,i5)') 'Vector s_i=H*b_i for i = ',j
          call printdmat(dgrad(1,j),nvar,1,text,'f20.12',ftape)
        end if

*================================================================
* 3) calculate approximate Hessian of lower dimension ("happ")
*================================================================

*----------------------------------------------------------------
* calculate H_approx_ij for all new possible values i,j
* H_approx_ij = s_i (dot) b_j
*----------------------------------------------------------------
        do i=1,j
          haij=dotprod(dgrad(1,i),normm(1,j),nvar)
          happ(i,j)=haij
          happ(j,i)=haij
        end do
      end do
      if(printall) then
        call flush(ftape)
        write(text,'(a,i5)') 'Approximate Hessian of dimension ',nbm
        call printdmat2(happ,nbm,maxnbm,nbm,text,'f20.12',ftape) 
        call flush(ftape)
      end if

*================================================================
* 4) diagonalize approximate Hessian; store eigenvalues and
*    eigenvectors
*================================================================
      if(nbm.gt.1) then
        do i=1,nbm
          do j=1,nbm
            evec(i,j)=happ(i,j)
          end do
        end do
*----------------------------------------
* determine optimum size of work array
*----------------------------------------
        lwork=(ilaenv(1,'DSYTRD','l',nbm,nbm,nbm,nbm)+2)*nvar
        allocate(work(lwork))
        call dsyev('V','L',nbm,evec,maxnbm,lam,work,lwork,info)
        deallocate(work)
        if(info.ne.0) call serror('davhes',2,'Could not '//
     *  'calculate eigenvectors##of approximate Hessian',ftape)
      else
        lam(1)=happ(1,1)
        evec(1,1)=1.0d0
      end if

*----------------------------------------------------------------
* print eigenvalues and eigenvectors for this iteration
*----------------------------------------------------------------
      if(printall) then
       write(text,'(a,i5)') 'Approximate eigenvalues for iteration ',iti
       call printdmat(lam,nbm,1,text,'f20.12',ftape) 
      end if
      write(text,'(a,i5)') 
     *  'Basis vector coefficients (eigenvectors) for iteration ',iti
      call printdmat2(evec,nbm,maxnbm,nbm,text,'f20.12',ftape) 

*----------------------------------------------------------------
* calculate approximate frequencies and print them
*----------------------------------------------------------------
      do i=1,nbm
        avib(i)=qfc2freq(lam(i))
      end do
      if(printall) then
        write(text, '(a,i5)') 
     *   'Approximate wavenumbers [1/cm] for iteration ',iti
        call printdmat(avib,nbm,1,text,'f20.12',ftape) 
      end if
      
*================================================================
* 5) calculate approximation for normal modes (vvec)
*================================================================
      do i=1,nbm
        do j=1,nvar
          vvec(j,i)=0.0d0
          do k=1,nbm
            vvec(j,i)=vvec(j,i)+evec(k,i)*normm(j,k)
          end do
        end do
      end do
      if(printall) then
        write(text,'(a,i5)')'Approximate normal modes in iteration ',iti
        call printdmat(vvec,nvar,nbm,text,'f20.12',ftape) 
      end if

*================================================================
* 6) calculate residuum vectors for all eigenvectors
*================================================================
      do j=1,nbm
        do i=1,nvar
          rvec(i,j)=0.0d0
          do k=1,nbm
            prec=evec(k,j)*(dgrad(i,k)-lam(j)*normm(i,k))
            rvec(i,j)=rvec(i,j)+prec
          end do
        end do
      end do
      if(printall) then
        write(text, '(a,i5)') 'Residuum vectors for iteration ',iti
        call printdmat(rvec,nvar,nbm,text,'f20.12',ftape) 
      end if

*================================================================
* 7) select eigenvector to be optimized (root homing)
*
* KK: here we explicitly assume that the number of vectors to be
*     optimized never increases
*================================================================
      do i=1,onsel
        oisel(i)=isel(i)
      end do
      do i=1,onsel
        if(.not.csel(i)) then
          call roothome(isel,iti,i,nbm,lam,olam,vvec,ovvec,normm,
     *     vtemp,rvec,onsel)
        end if
      end do

*--------------------------------------------------------------------
* calculate convergence criteria and check if convergence is achieved
*--------------------------------------------------------------------
      nsel2=0
      count=0
      dconv=.true.
*----------------------------------------------------------------
* there might be several roots to be determined
* (Davidson-Liu or block Davidson method)
*----------------------------------------------------------------
      do j=1,onsel
        call resicheck(isel,rvec,mxres,j,iti,evec,nbm,dcsel,onsel,
     *  olam,oisel,avib,norm,oldnorm)
        if(.not.csel(j)) then
          count=count+1
          csel(j)=dcsel
          if(csel(j)) then
            nsel2=nsel2+1
*----------------------------------------------------------------
* else: prepare calculation of next basis vectors
*----------------------------------------------------------------
          else
            ibm(count-nsel2)=j
            do i=1,nvar
              normm(i,nbm+count-nsel2)=rvec(i,isel(j))
            end do
          end if
          oldnorm(j)=norm
          write(ftape,*)
          dconv=(dconv.and.csel(j))
        end if
      end do

*----------------------------------------------------------------
* write elementary data to file
*----------------------------------------------------------------
      write(lunit,448) iti,(isel(j),avib(isel(j)),
     *  avib(isel(j))-qfc2freq(olam(oisel(j))),mxres(j),oldnorm(j),
     *  evec(nbm-(onsel-j),isel(j)),j=1,onsel)
      call flush(lunit)
448   format(i3,500(i9,f8.2,f10.2,2f12.8,f10.6))

*----------------------------------------------------------------
* store "old" eigenvalues and eigenvectors
*----------------------------------------------------------------
      do i=1,nbm
        olam(i)=lam(i)
        do j=1,nvar
          ovvec(j,i)=vvec(j,i)
        end do
      end do

*----------------------------------------------------------------
* set new no. of selected eigenvectors not yet converged
*----------------------------------------------------------------
      nsel=nsel-nsel2
	    
      if(dconv.or.nsel.eq.0) then
        write(ftape,'(2x,a)') 
        write(ftape,'(2x,a,i5)') 
     *     'Convergence is achieved in iteration ',iti
        write(ftape,'(2x,a)') 
        goto 900
      end if

*================================================================
* 8) Preconditioning: calculate new basis vectors
*
*    CAUTION: No. of new basis vectors can change between
*             two iterations (==> use ibm(j) instead of (j) for
*             the determination of the correct lambda for the
*             preconditioner)
*================================================================

      if((nbm+nsel).gt.maxnbm) return
*----------------------------------------------------------------
* allocate memory for matrix inversion (if requested)
* and matrix multiplication
*----------------------------------------------------------------
      allocate(work(nvar*nvar),iwork(nvar))
      lwork=(ilaenv(1,'DSYTRD','l',nvar,nvar,nvar,nvar)+2)*nvar
      if(nvar.gt.lwork) lwork=nvar
      allocate(work2(lwork))

*----------------------------------------------------------------
* do preconditioning for each selected vector (block Davidson)
*----------------------------------------------------------------
      do j=1,nsel
       icount=1
       work(:)=0.0d0
       if(aphs.and.(precond.eq.'aphsinv'.or.precond.eq.'jacobid')) then
*----------------------------------------------------------------
* A) use the inverse of (H_guess-lambda)_approx as preconditioner
*    (might be needed for Jacobi-Davidson also)
*----------------------------------------------------------------
         do i=1,nvar
           do k=1,nvar
            if(i.eq.k) then
              work((i-1)*nvar+k)=hguess(i,k)-lam(isel(ibm(j)))
            else
              work((i-1)*nvar+k)=hguess(i,k)
            end if
           end do
         end do
       else
         if(aphs.and.precond.eq.'aphsdia') then
*----------------------------------------------------------------
* B1) o.k., we've got an hessian guess from file
* => use diagonal elements as approximation (gives poor results)
*    (THIS IS ONLY A GOOD APPROXIMATION, IF THE MATRIX IS DIAGONAL
*     DOMINANT; THIS IS, IN GENERAL, NOT THE CASE FOR CARTESIAN
*     HESSIAN MATRICES, AND SHOULD ONLY BE USED IF NO APPROXIMATION
*     FOR THE COMPLETE HESSIAN IS AVAILABLE !!!)
*----------------------------------------------------------------
           do i=1,nvar
             work(i+(i-1)*nvar)=hguess(i,i)
           end do
         else if(precond.eq.'backtra'.and.nbm.ge.minnbt) then
*----------------------------------------------------------------
* B2) backtransformation; might be a very bad approximation,
* but accuracy of approximation increases as the number of
* basis vectors increases (usually better than B1)
*--------------------------------------------------------------------
* we want to approximate (H-lambda)^-1 
*--------------------------------------------------------------------
           do i=1,nvar
            do jj=1,nvar
             work(jj+(i-1)*nvar)=0.0d0
             do k=1,nbm
              do l=1,nbm
               work(jj+(i-1)*nvar)=work(jj+(i-1)*nvar)
     *                         +happ(k,l)*normm(jj,k)*normm(i,l)
              end do
             end do
            end do
           end do
         else 
*----------------------------------------------------------------
* B3) no idea at all; use unit matrix as hguess
*     (note that eigenvalue will be subtracted anyway)
*      new DEFAULT preconditioner (CH, 06/02/07)
*----------------------------------------------------------------
           if(precond.eq.'backtra') then
             write(idbg,*) '** unit matrix preconditioner used:'
             write(idbg,*) '** nbm, minnbt: ',nbm,minnbt
           end if
           do i=1,nvar
             work(i+(i-1)*nvar)=1.0d0
           end do
         end if
         if(printall) then
           call printdmat(work,nvar,nvar,'Hessian guess:','f22.11',idbg)
         end if
*--------------------------------------------------------------------
* ... it's 1/(H_ii - lambda), so subtract eigenvalue ...
* make sure that determinant is not too small
* (NOTE: this is not yet checked for the default preconditioner;
* but it will lead to a failure of the matrix inversion,
* which is checked anyway)
*--------------------------------------------------------------------
         do i=1,nvar
           work(i+(i-1)*nvar) = work(i+(i-1)*nvar) - lam(isel(ibm(j)))
           if(precond.eq.'unitmat'.or.precond.eq.'aphsdia'.or.
     &       (precond.eq.'backtra'.and.nbm.le.minnbt)) then
             if(abs(work(i+(i-1)*nvar)).lt.thres) 
     &         work(i+(i-1)*nvar)=1.0d0
           end if
         end do
       end if
       if(printall) then
         call printdmat(work,nvar,nvar,'(Hessian - lambda) guess:',
     *    'f22.11',idbg)
       end if
       call invhes(work,work2,lwork,iwork,nvar,info)
       if(printall) then
         call printdmat(work,nvar,nvar,'preconditioner:',
     *    'f22.11',idbg)
       end if

*-------------------------------------------------------------------
* for Jacobi-Davidson: 
* get value of epsilon (=> projection to orthogonal subspace; not
* clear what to do in Block-Davidson method)
* Note: at this point, normm(1,nbm+j) contains the residuum, not
*       the normal mode (should be identical to rvec(isel(ibm(j))))
*-------------------------------------------------------------------
       if(aphs.and.precond.eq.'jacobid') then
         epsilon=0.0d0
         call jacobid(normm(1,nbm+j),vvec(1,isel(ibm(j))),work,
     *     epsilon,nvar,vtemp,printall)
       end if

*----------------------------------------------------------------
* perform matrix - vector multiplication b_(i+1)= X r_i
* X = (H_guess - lambda)^-1 = preconditioner
*----------------------------------------------------------------
       call dgemm('N','N',nvar,1,nvar,1.0D0,work,nvar,
     *  normm(1,nbm+j),nvar,0.d0,vtemp,nvar)
       do i=1,nvar
         normm(i,nbm+j)=vtemp(i)
       end do

*-------------------------------------------------------------------
* for Jacobi-Davidson: 
* 1) perform matrix - vector multiplication t_(i+1)= X u_i
*    X = (H_guess - lambda)^-1 
* 2) calculate new basis vector b~_(i+1) = epsilon*t_(i+1) - b_(i+1)
*-------------------------------------------------------------------
       if(aphs.and.precond.eq.'jacobid') then
         call dgemm('N','N',nvar,1,nvar,1.0D0,work,nvar,
     *    vvec(1,isel(ibm(j))),nvar,0.d0,vtemp,nvar)
         do i=1,nvar
           normm(i,nbm+j)=epsilon*vtemp(i)-normm(i,nbm+j)
         end do
       end if

*--------------------------------------------------------------------
* normalization of new basis vector
*--------------------------------------------------------------------
       call normalize(normm(1,nbm+j),nvar)

*--------------------------------------------------------------------
* check overlap of basis vectors
*--------------------------------------------------------------------
       call calcovl(nvar,normm,normm,nbm+j,nbm+j,work)
       if(printall) then
         write(text, '(2(a,i5))') 
     *    'Overlap matrix (new vectors not orthonormalized)'
         call printdmat(work,nbm+j,nbm+j,text,'f20.12',ftape) 
       end if

*--------------------------------------------------------------------
* orthonormalization of new basis vector; skip basis vectors which
* are linearly dependent
*--------------------------------------------------------------------
       if(nsel.gt.0.and.printall) then
        write(text, '(2(a,i5))') 
     *   'Mass-weighted basis vectors (new vectors not orthogonalized)'
        call printdmat(normm(1,1),nvar,nbm+j,text,'f20.12',ftape) 
       end if
300    do i=1,nbm+j-1
         call gramschmidt(normm(1,i),normm(1,nbm+j),nvar,para,ovl,
     *     othres)
         write(idbg,'(a,i2,a,i2,a,f24.12)') 'overlap (',i,',',nbm+j,
     *     ') = ',ovl
         if(para) then
           write(ftape,'(2x,a)') 
     *      'Linear dependence detected for new basis vector'
           write(ftape,'(2x,a)') 
     *      'I will remove this vector from the basis set'
           write(ftape,*)
         end if
       end do

*--------------------------------------------------------------------
* check overlap with translational and rotational modes
*--------------------------------------------------------------------
       call calcovl(nvar,normm(1,nbm+j),trm,1,6,work)
       if(printall) then
         write(text, '(2(a,i5))') 
     *    'Overlap with transrot modes (not orthonormalized)'
         call printdmat(work,6,1,text,'f20.12',idbg) 
       end if

*---------------------------------------------------------------------
* in principle, we always want our basis vectors to be orthogonal
* to translational and rotational modes
* possible exceptions:
* - for testing and debugging
* - for linear molecules (since they are not treated in a proper way
*   otherwise)
* - to do a full frequency analysis with cartesian basis vectors
*   (although one can also in that case orthogonolize on trarot modes)
*---------------------------------------------------------------------
       if(.not.notrro) then
         do i=1,6
           call gramschmidt(trm(1,i),normm(1,nbm+j),nvar,para,ovl,
     *       othres)
           write(idbg,'(a,i2,a,i2,a,f24.12)') 'troverlap (',i,',',nbm+j,
     *     ') = ',ovl
           if(para) then
             write(ftape,'(2x,a)') 
     *        'Linear dependence detected for new basis vector'
             write(ftape,'(2x,a)') 
     *        'I will remove this vector from the basis set'
             write(ftape,*)
           end if
         end do

*--------------------------------------------------------------------
* check overlap with translational and rotational modes again
*--------------------------------------------------------------------
         call calcovl(nvar,normm(1,nbm+j),trm,1,6,work)
         if(printall) then
           write(text, '(2(a,i5))') 
     *      'Overlap with transrot modes (orthonormalized)'
           call printdmat(work,6,1,text,'f20.12',idbg) 
         end if
       end if
       if(nbmo.gt.0) then
         do i=1,nbmo
           call gramschmidt(ortmod(1,i),normm(1,nbm+j),nvar,para,ovl,
     *       othres)
           if(para) then
             write(ftape,'(2x,a)') 
     *        'Linear dependence with orthogonal subspace'
             write(ftape,'(2x,a)') 
     *        'I will remove this vector from the basis set'
             write(ftape,*)
           end if
         end do

*--------------------------------------------------------------------
* check overlap with orthogonal subspace
*--------------------------------------------------------------------
         call calcovl(nvar,normm(1,nbm+j),ortmod,1,nbmo,work)
         if(printall) then
           write(text, '(a)') 
     *      'Overlap with orthogonal subspace (orthonormalized)'
           call printdmat(work,nbmo,1,text,'f20.12',idbg) 
         end if
       end if

*--------------------------------------------------------------------
* check overlap of basis vectors again
*--------------------------------------------------------------------
       call calcovl(nvar,normm,normm,nbm+j,nbm+j,work)
       if(printall) then
         write(text, '(2(a,i5))') 
     *    'Overlap matrix (new vectors orthonormalized)'
         call printdmat(work,nbm+j,nbm+j,text,'f20.12',ftape) 
       end if
       call calcovl(nvar,normm(1,nbm+j),normm,1,nbm+j-1,work)
       if((.not.checkovl(work,nbm+j-1,othres)).and.
     *   (icount.le.iorth)) then
         write(ftape,'(2x,2(a,i6),a)') 
     *    'Next try to orthogonolize new vector no. ',j,' (count:',
     *     icount,')'
         icount=icount+1
         goto 300
       else if((.not.checkovl(work,nbm+j-1,othres)).and.
     *   (icount.gt.iorth)) then
         do i=1,nvar
           normm(i,nbm+j)=0.0d0
         end do
         write(ftape,'(2x,a)') 'Cannot orthogonalize new basis vector'
       end if
      end do
      call skipvec(normm,nvar,nbm+nsel,count,1.0d-2)
      nsel=nsel-count

*----------------------------------------------------------------
* deallocate memory for matrix inversion
*----------------------------------------------------------------
      deallocate(work,work2,iwork)

*----------------------------------------------------------------
* basis vector is mass-weighted, which is reversed here
*----------------------------------------------------------------
900   continue
      do i=1,maxnbm
        call nmod_m2c(normm(1,i),natoms,ic)
      end do
      if(nsel.gt.0.and.printall) then
        write(text, '(2(a,i5))') 
     *   'New cartesian basis vectors No. ',nbm+1,' - ',nbm+nsel
        call printdmat(normm(1,nbm+1),nvar,nsel,text,'f20.12',ftape) 
      end if
      return
      end

***********************************************************************

      subroutine d_grad(grad,dgrad,naus,nvar,cstepq,ino,maxnbm)

*--------------------------------------------------------------
* numerical differentiation of gradients w.r.t. a special mode
* of motion
*
* written by J. Neugebauer, Uni Erlangen, 2002
*--------------------------------------------------------------
#include "defs.h"
      integer naus,i,nvar,ino,maxnbm
      real*8 grad(nvar,naus,maxnbm),dgrad(nvar),cstepq,qfc2freq
      external qfc2freq
 
      if(naus.eq.2) then
       do i=1,nvar
        dgrad(i)=0.5d0/cstepq*(grad(i,1,ino)-grad(i,2,ino))
       end do
      else if(naus.eq.4) then
       do i=1,nvar
        dgrad(i)=1.0d0/(24.0d0*cstepq)* 
     *     (-2.0d0*grad(i,1,ino) 
     *      +16.0d0*grad(i,2,ino)
     *      -16.0d0*grad(i,3,ino)
     *      +2.0d0*grad(i,4,ino))
       end do
      else if(naus.eq.6) then
       do i=1,nvar
        dgrad(i)=1.0d0/(720.0d0*cstepq)*
     *     ( 12.0d0*grad(i,1,ino) 
     *     -108.0d0*grad(i,2,ino) 
     *     +540.0d0*grad(i,3,ino) 
     *     -540.0d0*grad(i,4,ino) 
     *     +108.0d0*grad(i,5,ino) 
     *      -12.0d0*grad(i,6,ino))
       end do 
      else
       call serror('d_grad',2,'naus > 6 not implemented',ftape)
      end if
      call flush(ftape)
      end    


***********************************************************************

      subroutine finalout(vvec,rvec,avib,iti,maxnbm,rthres,natoms,nbm,
     * precond,roothmpr,genstart,printall,rabsthres)
*----------------------------------------------------------------------
* write final output of frequencies, normal modes, and residuum vectors
* for all frequencies
*
* written by J. Neugebauer, Uni Erlangen, 2002
*----------------------------------------------------------------------
#include "defs.h"
      integer iti,maxnbm,natoms,nvar,nbm,inorm,i,j
      real*8 vvec(3*natoms,maxnbm),rvec(3*natoms,maxnbm),avib(maxnbm)
      real*8 rthres,dotprod,sqrt,norm,mxres,rabsthres
      logical genstart,printall,csel
      character*7 precond,roothmpr
      intrinsic sqrt
      external dotprod

      nvar=3*natoms
      write(ftape,*) 
      write(ftape,*) '***********************************************'
      if(precond.eq.'lanczos') then
        write(ftape,*) '**    Final results from module Lanczos      **'
      else
        write(ftape,*) '**    Final results from module Davidson     **'
      end if
      write(ftape,*) '***********************************************'
      write(ftape,*) 
      write(ftape,'(2x,a,i8)') 'No. of basis vectors used  : ',nbm
      write(ftape,'(2x,a,i8)') 'No. of iterations          : ',iti
      write(ftape,*) 
      inorm=0
      write(ftape,'(2x,a)') 
     *   'Mode No.  | Freq. [1/cm] ||r|/(3*natoms)| |r_max|      '//
     *   '| converged ?'
      write(ftape,'(2x,a)') 
     *   '----------|--------------|--------------|--------------'//
     *   '|------------'
      do i=1,nbm
        norm=sqrt(dotprod(rvec(1,i),rvec(1,i),nvar))
        norm=norm/(dble(3*natoms))
        mxres=0.0d0
        do j=1,nvar
          mxres=max(mxres,abs(rvec(j,i)))
        end do
        if(mxres.lt.rthres.and.(norm.lt.rabsthres.or.
     *     rabsthres.lt.0.0d0)) then
          inorm=inorm+1
          write(ftape,100) i,avib(i),norm,mxres,'YES'
        else
          write(ftape,100) i,avib(i),norm,mxres,' NO'
        end if
      end do
      write(ftape,*) 
      write(ftape,'(2x,a,i5)') 'No. of roots converged     : ',inorm
      write(ftape,'(2x,a,i5)') 'No. of roots not converged : ',nbm-inorm
      if(roothmpr.eq.'testmin') then
        if(avib(1).lt.-10.0d0) then
          write(ftape,'(2x,a)') 'Lowest eigenvalue < 0 ! Saddle point !'
        else if(avib(1).gt.10.0d0) then
          write(ftape,'(2x,a)') 'Lowest eigenvalue > 0 ! Minimum !'
        else
          write(ftape,'(2x,a)') 'Lowest eigenvalue close to zero !'
          write(ftape,'(2x,a)') 
     *           'Cannot decide whether this is a minimum or not !'
        end if
      end if
      if(printall) then
        write(ftape,'(2x,a)') 
        write(ftape,'(2x,a)') 'Printing eigenvectors for all roots ...'
        write(ftape,'(2x,a)') '---------------------------------------'
        write(ftape,'(2x,a)') 
        call printdmat(vvec,nvar,nbm,'Final eigenvectors','f20.12',
     *   ftape)
      end if
      if(genstart) then
        call serror('finalout',-1,'Note that this was a fake run##'//
     * 'to generate start vectors !',6)
        call serror('finalout',-1,'Note that this was a fake run##'//
     * 'to generate start vectors !',ftape)
      else
        if(inorm.eq.0) then
          call serror('finalout',1,'No roots converged!',6)
          call serror('finalout',1,'No roots converged!',ftape)
        else if(inorm.lt.nbm) then
          write(ftape,'(2x,a)') 
     *       'CAUTION! THERE ARE ROOTS NOT YET CONVERGED'
        end if
      end if
      return
100   format(2x,i9,' | ',f12.2,' | ',f12.8,' | ',f12.6,' | ',6x,a)
      end



***********************************************************************

      subroutine iterout(vvec,rvec,avib,iti,maxnbm,rthres,natoms,nbm,
     * precond,roothmpr,rabsthres)
*----------------------------------------------------------------------
* write intermediate output of frequencies, normal modes, and residuum 
* vectors for all frequencies
*
* written by J. Neugebauer, Uni Erlangen, 2002
*----------------------------------------------------------------------
#include "defs.h"
      integer iti,maxnbm,natoms,nvar,nbm,inorm,i,j
      real*8 vvec(3*natoms,maxnbm),rvec(3*natoms,maxnbm),avib(maxnbm)
      real*8 rthres,dotprod,sqrt,norm,mxres,rabsthres
      character*7 precond,roothmpr
      intrinsic sqrt
      external dotprod

      nvar=3*natoms
      write(ftape,*) 
      write(ftape,*)   '***********************************************'
      if(precond.eq.'lanczos') then
        write(ftape,*) '** Intermediate results from module Lanczos  **'
      else
        write(ftape,*) '** Intermediate results from module Davidson **'
      end if
      write(ftape,*)   '***********************************************'
      write(ftape,*) 
      write(ftape,'(2x,a,i8)') 'No. of basis vectors used  : ',nbm
      write(ftape,'(2x,a,i8)') 'No. of iterations          : ',iti
      write(ftape,*) 
      inorm=0
      write(ftape,'(2x,a)') 
     *   'Mode No.  | Freq. [1/cm] ||r|/(3*natoms)| |r_max|      '//
     *   '| converged ?'
      write(ftape,'(2x,a)') 
     *   '----------|--------------|--------------|--------------'//
     *   '|------------'
      do i=1,nbm
        norm=sqrt(dotprod(rvec(1,i),rvec(1,i),nvar))
        norm=norm/(dble(3*natoms))
        mxres=0.0d0
        do j=1,nvar
          mxres=max(mxres,abs(rvec(j,i)))
        end do
        if(mxres.lt.rthres.and.(norm.lt.rabsthres.or.
     *     rabsthres.lt.0.0d0)) then
          inorm=inorm+1
          write(ftape,100) i,avib(i),norm,mxres,'YES'
        else
          write(ftape,100) i,avib(i),norm,mxres,' NO'
        end if
      end do
      write(ftape,*) 
      write(ftape,'(2x,a,i5)') 'No. of roots converged     : ',inorm
      write(ftape,'(2x,a,i5)') 'No. of roots not converged : ',nbm-inorm
      write(ftape,*) 
      if(roothmpr.eq.'testmin') then
        if(avib(1).lt.-10.0d0) then
          write(ftape,'(2x,a)') 'Lowest eigenvalue < 0 ! Saddle point !'
        else if(avib(1).gt.10.0d0) then
          write(ftape,'(2x,a)') 'Lowest eigenvalue > 0 ! Minimum !'
        else
          write(ftape,'(2x,a)') 'Lowest eigenvalue close to zero !'
          write(ftape,'(2x,a)') 
     *           'Cannot decide whether this is a minimum or not !'
        end if
      end if
      call flush(ftape)
      return
100   format(2x,i9,' | ',f12.2,' | ',f12.8,' | ',f12.6,' | ',6x,a)
      end



******************************************************************
 
      subroutine wfreq(nvar,vibf,lu)
*
* write vibrational wavenumbers to unit <lu> in Turbomole format
*
* written by J. Neugebauer, Uni Erlangen, 2002
*
      implicit none
      integer nvar,lu,i
      real*8 vibf(nvar)
      write(lu,'(A)') '$vibrational spectrum'
      write(lu,'(A)') '#  mode     symmetry     wave number   IR'
     *//' intensity    selection rules'
      write(lu,'(A)') '#                         cm**(-1)       '
     *//' km/mol         IR     RAMAN'
      do i=1,nvar
        write(lu,'(i6,8x,a4,f16.2)') i,'   a',vibf(i)
      end do
      end            
