
      subroutine getstrbm(coord,natoms,normm,ic,at1,at2,nbm)

*-------------------------------------------------------------------
* construction of a normalized vibrational stretching mode
* taking into account different masses of the atoms.
* for a diatomic molecule, this would be the exact solution
*
* for more details see Wilson, Decius, Cross, Molecular Vibrations, 
* McGraw-Hill, New York, 1955, Section 4.1 (p. 55-56)
*
* written by J. Neugebauer, Uni Erlangen, 2002
*-------------------------------------------------------------------
      implicit none
      integer at1,at2,natoms,i,j,ic(natoms),nbm
      real*8 coord(3,natoms),normm(3*natoms,nbm),find_mass,m1,m2
      real*8 checkcom
      external find_mass,checkcom

*-------------------------------------------------------------------
* set up normal mode for corresponding diatomic molecule
* (contributions from atoms i,j are proportional to m_{j,i}/m_total)
*-------------------------------------------------------------------
      m1=find_mass(ic(at1))
      m2=find_mass(ic(at2))
      do i=1,3
        normm((at1-1)*3+i,nbm)=(coord(i,at1)-coord(i,at2))*m2
        normm((at2-1)*3+i,nbm)=(coord(i,at2)-coord(i,at1))*m1
      end do

*------------------------------------------------------------
* renormalize
*------------------------------------------------------------
      call normalize(normm(1,nbm),3*natoms)
      write(*,*) 'Center of mass distortion: ',
     *   checkcom(normm(1,nbm),natoms,ic)
      end

*******************************************************************

      subroutine getbenbm(coord,natoms,normm,ic,at1,at2,at3,nbm)

*-------------------------------------------------------------------
* set up bending mode for atoms   2
*                                / \
*                               1   3
*
* for details see Wilson, Decius, Cross, Molecular Vibrations, 
* McGraw-Hill, New York, 1955, Section 4.1 (p. 56-58)
* 
* written by J. Neugebauer, Uni Erlangen, 2002
*-------------------------------------------------------------------
      implicit none
#include "fconstants.h"
      integer at1,at2,natoms,i,j,ic(natoms),nbm,at3
      real*8 coord(3,natoms),normm(3*natoms,nbm),find_mass,m1,m2
      real*8 m3,rstart(3),r1(3),r2(3),r3(3),r21(3),r23(3),norm
      real*8 thres,dotprod,d12,d23,a123,sin,cos,norm3d,anglxy
      real*8 checkcom,sina123,cosa123,ovl
      parameter(thres=1.0d-6)
      logical para
      intrinsic sin,cos
      external find_mass,dotprod,checkcom,norm3d,anglxy

      m1=find_mass(ic(at1))
      m2=find_mass(ic(at2))
      m3=find_mass(ic(at3))

*------------------------------------------------------------
* calculate r21 and r23
*------------------------------------------------------------
      do i=1,3
        r21(i)=(coord(i,at1)-coord(i,at2))
        r23(i)=(coord(i,at3)-coord(i,at2))
      end do
      d12=norm3d(r21)
      d23=norm3d(r23)

*------------------------------------------------------------
* normalize vectors r21 and r23
*------------------------------------------------------------
      call normalize3d(r21)
      call normalize3d(r23)
      
*------------------------------------------------------------
* calculate angle a123 = phi (Wilson et al.)
* and sina123=sin(phi) 
* Note: r21 =   r31 (wilson)
*       r23 =   r32 (wilson)
*------------------------------------------------------------
      a123=anglxy(r21,r23)
      sina123=sin(a123)
      cosa123=cos(a123)

      if(abs(sina123).gt.thres) then
        do i=1,3
          r1(i)=(cosa123*r21(i)-r23(i))/d12/sina123
          r2(i)=((d12-d23*cosa123)*(r21(i))+(d23-d12*cosa123)*r23(i))
          r2(i)=r2(i)/d12/d23/sina123
          r3(i)=(cosa123*r23(i)-r21(i))/d23/sina123
        end do
      else
        do i=1,3
          r1(i)=1.0d0
        end do
        call gramschmidt(r21,r1,3,para,ovl,thres)
        do i=1,3
          r3(i)=r1(i)
          r2(i)=-r1(i)-r3(i)
        end do
      end if

*------------------------------------------------------------
* mass-weighting to conserve center of mass (???)
*------------------------------------------------------------
      do i=1,3
        r1(i)=r1(i)/m1
        r2(i)=r2(i)/m2
        r3(i)=r3(i)/m3
      end do

      do i=1,3
        normm((at1-1)*3+i,nbm)=r1(i)
        normm((at2-1)*3+i,nbm)=r2(i)
        normm((at3-1)*3+i,nbm)=r3(i)
      end do

*------------------------------------------------------------
* renormalize
*------------------------------------------------------------
      call normalize(normm(1,nbm),3*natoms)
      write(*,*) 'Center of mass distortion: ',
     *   checkcom(normm(1,nbm),natoms,ic)
      end

*******************************************************************

      logical function checkat(at1,at2,atb,atb2,at3,atb3,nbm,btyp,atyp)
*
* check if basis mode is already defined for these atoms
*
* written by J. Neugebauer, Uni Erlangen, 2002
*
      implicit none
      integer i,nbm,atyp
      integer at1,at2,atb(nbm),atb2(nbm),btyp(nbm),at3,atb3(nbm)
      logical ateq
      ateq=.false.
      checkat=.false.
      do i=1,nbm
       if(atyp.eq.btyp(i)) then
        if(atyp.eq.1) then
          if((at1.eq.atb(i).and.at2.eq.atb2(i)).or.
     *     (at1.eq.atb2(i).and.at2.eq.atb(i))) checkat=.true.
        else if(atyp.eq.5) then
          if((at1.eq.atb(i).and.at2.eq.atb2(i).and.at3.eq.atb3(i)).or.
     *     (at1.eq.atb3(i).and.at2.eq.atb2(i).and.at3.eq.atb(i)))
     *      checkat=.true.
        else
          if(at1.eq.atb(i)) checkat=.true.
        end if
       end if
      end do
      end 

*******************************************************************

      subroutine gettorsbm(coord,natoms,normm,ic,at1,at2,at3,at4,nbm)

*-------------------------------------------------------------------
* set up torsion mode for atoms   2 - 3
*                                /     \
*                               1       4
*
* for details see Wilson, Decius, Cross, Molecular Vibrations, 
* McGraw-Hill, New York, 1955, Section 4.1 (p. 60-61)
* (Note: might there be an error in the sign of the second term
* for s_t2 and s_t3 in Wilson et al. ?).
* 
* written by J. Neugebauer, Uni Erlangen, 2002
*-------------------------------------------------------------------
      implicit none
#include "fconstants.h"
      integer at1,at2,natoms,i,j,ic(natoms),nbm,at3,at4
      real*8 coord(3,natoms),normm(3*natoms,nbm),find_mass,m1,m2
      real*8 m3,rstart(3),r1(3),r2(3),r3(3),r12(3),r23(3),norm,m4
      real*8 thres,dotprod,r34(3),a123,a234,rscr(3),rscr2(3),r4(3)
      real*8 d12,d23,d34,sina123,sina234,sin,cos,cosa123,cosa234
      real*8 norm3d,anglxy,checkcom
      parameter(thres=1.0d-6)
      logical para
      external find_mass,dotprod,norm3d,anglxy,checkcom
      intrinsic sin,cos

      m1=find_mass(ic(at1))
      m2=find_mass(ic(at2))
      m3=find_mass(ic(at3))
      m4=find_mass(ic(at4))

*------------------------------------------------------------
* calculate vectors r12, r23, and r34
* calculate distances d12, d23, d34
*------------------------------------------------------------
      do i=1,3
        r12(i)=(coord(i,at2)-coord(i,at1))
        r23(i)=(coord(i,at3)-coord(i,at2))
        r34(i)=(coord(i,at4)-coord(i,at3))
      end do
      d12=norm3d(r12)
      d23=norm3d(r23)
      d34=norm3d(r34)

*------------------------------------------------------------
* normalize vectors r12, r23, and r34
*------------------------------------------------------------
      call normalize3d(r12)
      call normalize3d(r23)
      call normalize3d(r34)

*------------------------------------------------------------
* calculate angles a123 = phi_2 (Wilson et al.)
*              and a234 = phi_3 (Wilson et al.)
*------------------------------------------------------------
      do i=1,3
        rscr(i)=-r12(i)
        rscr2(i)=-r23(i)
      end do
      a123=anglxy(rscr,r23)
      a234=anglxy(rscr2,r34)
      sina123=sin(a123)
      sina234=sin(a234)
      cosa123=cos(a123)
      cosa234=cos(a234)

*------------------------------------------------------------
* calculate vector r1 = s_t1 (see Wilson et al.)
*------------------------------------------------------------
      call vecprod(r12,r23,rscr)
      do i=1,3
        r1(i)=-rscr(i)/d12/sina123/sina123
      end do

*------------------------------------------------------------
* calculate vector r4 = s_t4 (see Wilson et al.)
*------------------------------------------------------------
      call vecprod(r34,r23,rscr2)
      do i=1,3
        r4(i)=-rscr2(i)/d34/sina234/sina234
      end do

*------------------------------------------------------------
* calculate vector r2 = s_t2 (see Wilson et al.)
*------------------------------------------------------------
      do i=1,3
        r2(i)=(d23-d12*cosa123)/d23/d12/sina123/sina123*rscr(i)
        r2(i)=r2(i)+(cosa234/d23/sina234/sina234*rscr2(i))
CError in Wilson ?  Wilson:      r2(i)=r2(i)-(cosa234/d23/sina234/sina234*rscr2(i))
      end do

*------------------------------------------------------------
* calculate vector r3 = s_t3 (see Wilson et al.)
*------------------------------------------------------------
      do i=1,3
        r3(i)=(d23-d34*cosa234)/d23/d34/sina234/sina234*rscr2(i)
        r3(i)=r3(i)+(cosa123/d23/sina123/sina123*rscr(i))
CError in Wilson ?  Wilson:      r3(i)=r3(i)-(cosa123/d23/sina123/sina123*rscr(i))
C if this is applied, r1+r2+r3+r4 >< 0 !!!
      end do

*------------------------------------------------------------
* mass-weighting to conserve center of mass (???)
*------------------------------------------------------------
      do i=1,3
        r1(i)=r1(i)/m1
        r2(i)=r2(i)/m2
        r3(i)=r3(i)/m3
        r4(i)=r4(i)/m4
      end do

      do i=1,3
        normm((at1-1)*3+i,nbm)=r1(i)
        normm((at2-1)*3+i,nbm)=r2(i)
        normm((at3-1)*3+i,nbm)=r3(i)
        normm((at4-1)*3+i,nbm)=r4(i)
      end do

*------------------------------------------------------------
* renormalize
*------------------------------------------------------------
      call normalize(normm(1,nbm),3*natoms)
      write(*,*) 'Center of mass distortion: ',
     *   checkcom(normm(1,nbm),natoms,ic)
      end

*******************************************************************

      subroutine getoopbm(coord,natoms,normm,ic,at1,at2,at3,at4,nbm)

*-------------------------------------------------------------------
* set up out-of-plane mode for atoms       3 
*                                         / 
*                                    1 - 4 
*                                         \
*                                          2
*
* where the plane is defined by atoms 2, 3, and 4, and the motion
* changes the angle theta between the bond 1 - 4 and this plane.
* 
*
* for details see Wilson, Decius, Cross, Molecular Vibrations, 
* McGraw-Hill, New York, 1955, Section 4.1 (p. 58-60)
* 
* written by J. Neugebauer, Uni Erlangen, 2002
*-------------------------------------------------------------------
      implicit none
#include "fconstants.h"
      integer at1,at2,natoms,i,j,ic(natoms),nbm,at3,at4
      real*8 coord(3,natoms),normm(3*natoms,nbm),find_mass,m1,m2
      real*8 m3,rstart(3),r1(3),r2(3),r3(3),r41(3),r42(3),norm,m4
      real*8 thres,dotprod,r43(3),phi1,phi2,rscr(3),rscr2(3),r4(3)
      real*8 d14,d24,d34,sphi1,sphi2,phi3,sin,cos,ctheta,tan,asin
      real*8 norm3d,anglxy,checkcom,theta,ttheta,sphi3,rscr3(3)
      real*8 scalar3d,cphi1,cphi2,cphi3,rad2deg
      parameter(thres=1.0d-6)
      logical para
      external find_mass,dotprod,norm3d,anglxy,checkcom,scalar3d
      intrinsic sin,cos,tan,asin

      rad2deg=360.0d0/2.0d0/pi
      m1=find_mass(ic(at1))
      m2=find_mass(ic(at2))
      m3=find_mass(ic(at3))
      m4=find_mass(ic(at4))

*------------------------------------------------------------
* calculate vectors r41, r42, r43
* calculate distances d14, d24, d34
*------------------------------------------------------------
      do i=1,3
        r41(i)=(coord(i,at1)-coord(i,at4))
        r42(i)=(coord(i,at2)-coord(i,at4))
        r43(i)=(coord(i,at3)-coord(i,at4))
      end do
      d14=norm3d(r41)
      d24=norm3d(r42)
      d34=norm3d(r43)

*------------------------------------------------------------
* normalize vectors 
*------------------------------------------------------------
      call normalize3d(r41)
      call normalize3d(r42)
      call normalize3d(r43)

*------------------------------------------------------------
* calculate angles a243 = phi_1 (Wilson et al.),
*                  a143 = phi_2 (Wilson et al.),
*              and a142 = phi_3 (Wilson et al.)
*------------------------------------------------------------
      phi1=anglxy(r42,r43)
      phi2=anglxy(r41,r42)
      phi3=anglxy(r43,r41)
      sphi1=sin(phi1)
      sphi2=sin(phi2)
      sphi3=sin(phi3)
      cphi1=cos(phi1)
      cphi2=cos(phi2)
      cphi3=cos(phi3)

*------------------------------------------------------------
* calculate vector rscr  = e42 x e43
*                  rscr2 = e43 x e41
*                  rscr3 = e41 x e42
*------------------------------------------------------------
      call vecprod(r42,r43,rscr)
      call vecprod(r43,r41,rscr2)
      call vecprod(r41,r42,rscr3)

*------------------------------------------------------------
* calculate theta, cos(theta)
*------------------------------------------------------------
      theta=scalar3d(rscr,r41)/sphi1
      theta=asin(theta)
      ctheta=cos(theta)
      ttheta=tan(theta)

*------------------------------------------------------------
* calculate vector r1 = s_t1 (see Wilson et al.)
*------------------------------------------------------------
      do i=1,3
        r1(i)=(rscr(i)/ctheta/sphi1-ttheta*r41(i))/d14
      end do

*------------------------------------------------------------
* calculate vector r2 = s_t2 (see Wilson et al.)
*------------------------------------------------------------
      do i=1,3
        r2(i)=(rscr2(i)/ctheta/sphi1-ttheta/sphi1/sphi1*
     *    (r42(i)-cphi1*r43(i)))/d24
      end do

*------------------------------------------------------------
* calculate vector r3 = s_t3 (see Wilson et al.)
*------------------------------------------------------------
      do i=1,3
        r3(i)=(rscr3(i)/ctheta/sphi1-ttheta/sphi1/sphi1*
     *    (r43(i)-cphi1*r42(i)))/d34
      end do

*------------------------------------------------------------
* calculate vector r4 = s_t4 (see Wilson et al.)
*------------------------------------------------------------
      do i=1,3
        r4(i)=-r1(i)-r2(i)-r3(i)
      end do

*------------------------------------------------------------
* mass-weighting to conserve center of mass (???)
*------------------------------------------------------------
      do i=1,3
        r1(i)=r1(i)/m1
        r2(i)=r2(i)/m2
        r3(i)=r3(i)/m3
        r4(i)=r4(i)/m4
      end do

      do i=1,3
        normm((at1-1)*3+i,nbm)=r1(i)
        normm((at2-1)*3+i,nbm)=r2(i)
        normm((at3-1)*3+i,nbm)=r3(i)
        normm((at4-1)*3+i,nbm)=r4(i)
      end do

*------------------------------------------------------------
* renormalize
*------------------------------------------------------------
      call normalize(normm(1,nbm),3*natoms)
      write(*,*) 'Center of mass distortion: ',
     *   checkcom(normm(1,nbm),natoms,ic)
      end

*************************************************************************

      subroutine breathm(coord,ic,normm,natoms,mflag)

*-------------------------------------------------------------------
* use (mass-weighted) coordinate vectors as normal coordinate 
* vectors => totally symmetric breathing mode
*
* sum_i q^(c)_i * m_i = 0  => q^(c)_i m_i = const (for all i)
*                          => q^(c)_i = const/m_i (unnormalized)
* 
* written by J. Neugebauer, Uni Erlangen, 2002
*-------------------------------------------------------------------
      implicit none
      integer i,j,natoms,ic(natoms),nvar
      real*8 normm(3,natoms),coord(3,natoms),com(3),find_mass
      real*8 tmass
      logical mflag
      external find_mass
      nvar=3*natoms

*******
* determine center of mass
*******
      call getcom(natoms,coord,ic,com)

*******
* move origin to center of mass
*******
      if(mflag) then
        do i=1,natoms
          do j=1,3
            normm(j,i)=(coord(j,i)-com(j))/find_mass(ic(i))
          end do
        end do
      else
        do i=1,natoms
          do j=1,3
            normm(j,i)=(coord(j,i)-com(j))
          end do
        end do
      end if

******
* normalize
******
      call normalize(normm,nvar)

      end
