      module MachineModule

!     =============================================================
!     module to handle the parallel virtual machine
!
!     written by J. Neugebauer, 2007
!
!     still missing: add_node should always check if there are
!                    nodes with mach%nodf = 'remo'
!                 => these nodes can be overwritten in the nodelist
!
!     CAUTION: machine-pointer can only directly be passed from
!              one routine to the other if the other is part of
!              a proper module with properly defined interface
!              (otherwise, strange things happen, e.g., what is
!              passed on is not mach but mach%next ...)
!
!     TO CIRCUMVENT this problem just pass mach%numb to the
!     subroutine and call findnode_by_num(mymach,numb) from there
!     to get back the current machine pointer
!     =============================================================

      implicit none
      public

      type type_machine
        type(type_machine), pointer :: next
        type(type_machine), pointer :: endoflist
        integer                     :: tid
        integer                     :: tpid
        integer                     :: time
        integer                     :: tmsg
        integer                     :: trc
        integer                     :: usag
        integer                     :: stpa
        integer                     :: stpb
        integer                     :: numb
        integer                     :: no
        integer                     :: mxlo
        integer                     :: load
        integer                     :: last
        integer                     :: res
        character*4                 :: nodf
        character*4                 :: cmd
        character*40                :: name
        character*40                :: pmsg
        character*255               :: arch
        character*255               :: comment
      end type type_machine

!     ----------------------------------------------------
!     global pointer to machine lists and current machine
!     machlist: list of machines found in USE_NODES
!               (one for each process to be spawned)
!     pvmmachlist: list of machines in PVM
!               (one for each node)
!     ----------------------------------------------------
      type(type_machine), pointer, save :: machlist
      type(type_machine), pointer, save :: pvmmachlist
      type(type_machine), pointer, save :: current_mach

!     ----------------------------------------------------
!     signals for subroutine remnode
!     ----------------------------------------------------
      integer, parameter                :: mach_term = 902
      integer, parameter                :: mach_fai1 = -8
      integer, parameter                :: mach_fai2 = -7
      integer, parameter                :: mach_shut = 901

      integer, private, parameter :: idbg = 41
      integer, private, save      :: numb = 0

      contains

!*****************************************************************************

      subroutine init_machlist
        nullify(machlist)
        nullify(pvmmachlist)
        nullify(current_mach)
      end subroutine init_machlist

!*****************************************************************************

      subroutine add_node(list,name,arch)

!     ===============================================
!     add new node to <list> of nodes
!     ===============================================
        type(type_machine), pointer     :: list
        type(type_machine), pointer     :: mach,oldend
        integer                         :: no
        character*(*)                   :: name
        character*(*)                   :: arch

!       ---------------------------------------------------
!       get number of this machine in list and in all lists
!       ---------------------------------------------------
        numb = numb + 1
        no = 0
        nullify(mach)
        mach => list
        do while(associated(mach))
          no = mach%no
          mach => mach%next
        end do
        no = no + 1
        write(idbg,*) 'add_node | adding node no. ',numb
        write(idbg,*) 'add_node | no. in this list',no

!       ---------------------------------------------
!       create new entry in list
!       ---------------------------------------------
        nullify(mach)
        allocate(mach)
        mach%name = trim(name)
        mach%arch = trim(arch)
        call clean(mach%pmsg)
        call clean(mach%comment)
        mach%time = 0
        mach%tmsg = 0
        mach%tid  = 0
        mach%tpid = 0
        mach%trc  = 0
        mach%usag = 0
        mach%nodf = 'wait'
        mach%stpa = 0
        mach%stpb = 0
        mach%res  = 0
        mach%mxlo = 0
        mach%load = 0
        mach%cmd  = '----'
        mach%last = 0
        mach%numb = numb
        mach%no   = no
        nullify(mach%endoflist)
        nullify(mach%next)

        write(idbg,*) 'add_node | enqueuing node in list'
!       ---------------------------------------------
!       keep list as a pointer to beginning of list,
!       insert mach at end of list
!       ---------------------------------------------
        if(.not.associated(list)) then
          list => mach
          write(idbg,*) 'add_node | list => node'
          nullify(list%endoflist)
          nullify(list%next)
        end if
        if(.not.associated(list%endoflist)) then
          list%endoflist => list
          write(idbg,*) 'add_node | list%endoflist => list'
          nullify(list%endoflist%next)
          write(idbg,*) 'add_node | nullified: list%endoflist'
        else
          nullify(oldend)
          oldend => list%endoflist
          list%endoflist => mach
          oldend%next => list%endoflist
        end if
        current_mach => mach
      end subroutine add_node

!*******************************************************************************

      subroutine findnode_by_tid(cmach,tid)
!     ===================================================
!     find node in <machlist> by task ID
!     ===================================================
        type(type_machine), pointer     :: cmach
        type(type_machine), pointer     :: mach
        integer                         :: tid

        nullify(mach)
        nullify(cmach)
        mach => machlist
        do while(associated(mach))
          if(mach%tid == tid) then
            current_mach => mach
            cmach => mach
            return
          end if
          mach => mach%next
        end do

      end subroutine findnode_by_tid

!*******************************************************************************

      logical function mach_ok(mach)
!     ===================================================
!     points to <current machine>
!     ===================================================
        type(type_machine), pointer     :: mach

        mach_ok = .true.
        mach_ok = mach_ok .and. (mach%nodf /= 'dead')
        mach_ok = mach_ok .and. (mach%nodf /= 'remo')
        mach_ok = mach_ok .and. (mach%nodf /= 'fai1')
        mach_ok = mach_ok .and. (mach%nodf /= 'fai2')
        mach_ok = mach_ok .and. (mach%nodf /= 'pvme')
        mach_ok = mach_ok .and. (mach%nodf /= 'virt')
        mach_ok = mach_ok .and. (mach%nodf /= 'nans')
        mach_ok = mach_ok .and. (mach%nodf /= 'unf ')
        mach_ok = mach_ok .and. (mach%nodf /= 'chk ')
        mach_ok = mach_ok .and. (mach%nodf /= 'done')

      end function mach_ok

!*******************************************************************************

      subroutine setmach_pmsg(mach,message)
!     ===================================================
!     set machine process message
!     ===================================================
        type(type_machine), pointer     :: mach
        character*(*)                   :: message
        integer                         :: lmessage,i

        lmessage = len_trim(message)
        call clean(mach%pmsg)
        i = min(lmessage,len(mach%pmsg))
        mach%pmsg(1:i) = message(1:i)

      end subroutine setmach_pmsg

!*******************************************************************************

      subroutine setmach_name(mach,name)
!     ===================================================
!     set machine name
!     ===================================================
        type(type_machine), pointer     :: mach
        character*(*)                   :: name
        integer                         :: lname,i

        lname = len_trim(name)
        call clean(mach%name)
        i = min(lname,len(mach%name))
        mach%name(1:i) = name(1:i)

      end subroutine setmach_name

!*******************************************************************************

      subroutine getmach_name(mach,name)
!     ===================================================
!     return name of this machine
!     ===================================================
        type(type_machine), pointer     :: mach
        character*(*)                   :: name
        integer                         :: lname,i

        lname = len_trim(mach%name)
        call clean(name)
        i = min(lname,len(name))
        name(1:i) = mach%name(1:i)

      end subroutine getmach_name

!*******************************************************************************

      subroutine getmach_pmsg(mach,message)
!     ===================================================
!     return process message for this machine
!     ===================================================
        type(type_machine), pointer     :: mach
        character*(*)                   :: message
        integer                         :: lmessage,i

        lmessage = len_trim(mach%pmsg)
        call clean(message)
        i = min(lmessage,len(message))
        message(1:i) = mach%pmsg(1:i)

      end subroutine getmach_pmsg


!*******************************************************************************

      subroutine findnode_by_num(cmach,numb)
!     ===================================================
!     find node in <machlist> by number
!     ===================================================
        type(type_machine), pointer     :: cmach
        type(type_machine), pointer     :: mach
        integer                         :: numb

        nullify(mach)
        nullify(cmach)
        mach => machlist
        do while(associated(mach))
          if(mach%numb == numb) then
            current_mach => mach
            cmach => mach
            return
          end if
          mach => mach%next
        end do

      end subroutine findnode_by_num

!*******************************************************************************

      subroutine findnode_by_name(list,cmach,name)
!     ===================================================
!     find node in <list> by name;
!     return first entry which matches name
!     ===================================================
        type(type_machine), pointer     :: list
        type(type_machine), pointer     :: cmach
        type(type_machine), pointer     :: mach
        character*(*)                   :: name

        nullify(mach)
        nullify(cmach)
        mach => list
        do while(associated(mach))
          if(mach%name == name) then
            current_mach => mach
            cmach => mach
            return
          end if
          mach => mach%next
        end do

      end subroutine findnode_by_name

!*******************************************************************************

      subroutine print_node(mach,io)
        type(type_machine), pointer     :: mach
        integer                         :: io

        write(io,'(2x,a,i5)') '----------------------------------------'
        write(io,'(2x,a,i5)') 'MACHINE STRUCTURE'
        write(io,'(2x,a,i5)') '----------------------------------------'

        if(associated(mach)) then
          write(io,'(2x,2a)') 'MACHINE name         : ',trim(mach%name)
          write(io,'(2x,2a)') '        architecture : ',trim(mach%arch)
          write(io,'(2x,2a)') '        proc. mess.  : ',trim(mach%pmsg)
          write(io,'(2x,a,i10)') '        numb         : ',mach%numb
          write(io,'(2x,a,i10)') '        time         : ',mach%time
          write(io,'(2x,a,i10)') '        tmsg         : ',mach%tmsg
          write(io,'(2x,a,i10)') '        tid          : ',mach%tid
          write(io,'(2x,a,i10)') '        tpid         : ',mach%tpid
          write(io,'(2x,a,i10)') '        trc          : ',mach%trc
          write(io,'(2x,a,i10)') '        usag         : ',mach%usag
          write(io,'(2x,2a)') '        nodf         : ',trim(mach%nodf)
          write(io,'(2x,a,i10)') '        stpa         : ',mach%stpa
          write(io,'(2x,a,i10)') '        stpb         : ',mach%stpb
          write(io,'(2x,a,i10)') '        res          : ',mach%res
          write(io,'(2x,a,i10)') '        mxlo         : ',mach%mxlo
          write(io,'(2x,a,i10)') '        load         : ',mach%load
          write(io,'(2x,2a)') '        cmd          : ',trim(mach%cmd)
          write(io,'(2x,a,i10)') '        last         : ',mach%last
          write(io,'(2x,2a)') &
     &        '        comment      : ',trim(mach%comment)
        end if

        write(io,'(2x,a,i5)') '----------------------------------------'

      end subroutine print_node

!*******************************************************************************

      subroutine print_nodes(list,io)
        type(type_machine), pointer     :: list
        type(type_machine), pointer     :: mach
        integer                         :: io

        write(io,'(2x,a,i5)') '----------------------------------------'
        write(io,'(2x,a,i5)') 'List of machines: '
        write(io,'(2x,a,i5)') '----------------------------------------'

        nullify(mach)
        mach => list
        do while(associated(mach))
          write(io,'(2x,2a)') 'MACHINE name         : ',trim(mach%name)
          write(io,'(2x,2a)') '        architecture : ',trim(mach%arch)
          write(io,'(2x,2a)') '        proc. mess.  : ',trim(mach%pmsg)
          write(io,'(2x,a,i10)') '        numb         : ',mach%numb
          write(io,'(2x,a,i10)') '        time         : ',mach%time
          write(io,'(2x,a,i10)') '        tmsg         : ',mach%tmsg
          write(io,'(2x,a,i10)') '        tid          : ',mach%tid
          write(io,'(2x,a,i10)') '        tpid         : ',mach%tpid
          write(io,'(2x,a,i10)') '        trc          : ',mach%trc
          write(io,'(2x,a,i10)') '        usag         : ',mach%usag
          write(io,'(2x,2a)') '        nodf         : ',trim(mach%nodf)
          write(io,'(2x,a,i10)') '        stpa         : ',mach%stpa
          write(io,'(2x,a,i10)') '        stpb         : ',mach%stpb
          write(io,'(2x,a,i10)') '        res          : ',mach%res
          write(io,'(2x,a,i10)') '        mxlo         : ',mach%mxlo
          write(io,'(2x,a,i10)') '        load         : ',mach%load
          write(io,'(2x,2a)') '        cmd          : ',trim(mach%cmd)
          write(io,'(2x,a,i10)') '        last         : ',mach%last
          write(io,'(2x,2a)') &
     &        '        comment      : ',trim(mach%comment)
          mach => mach%next
        end do

        write(io,'(2x,a,i5)') '----------------------------------------'

      end subroutine print_nodes

      subroutine machtest(mach)
      type(type_machine), pointer :: mach
      write(*,*) 'mach%numb: ',mach%numb
      write(*,*) 'mach%next%numb: ',mach%next%numb
      end subroutine machtest

      end module MachineModule
