***********************************************************************

      subroutine trarot(co,tmodes,rmodes,ic,natoms)

*----------------------------------------------------------------------
* set up pure translational and rotational modes
* 
* written by J. Neugebauer, Uni Erlangen, 2002
*----------------------------------------------------------------------
#include "defs.h"

      real*8, allocatable :: nco(:)
      integer i,natoms
      integer ic(natoms)
      real*8 inertia(3,3),ieigv(3),co(3,natoms),tmodes(3*natoms,3)
      real*8 rmodes(3*natoms,3)
      logical prnmod
 
      prnmod=.false.

*-------------------------------------------------------
* allocate memory for and set up coordinates in center-
* of-mass coordinate system
*-------------------------------------------------------

      allocate(nco(3*natoms))
      call move2cm(co,natoms,ic,nco)

*--------------------------------------------------------------
* transform coordinates to principal axes of moments of inertia
*--------------------------------------------------------------

      call pamitrans(nco,natoms,ic,inertia,ieigv)

*--------------------------------------------------------------
* set up translational and rotational modes
*--------------------------------------------------------------

      call gettrans(natoms,ic,tmodes)
      call getrot(nco,natoms,ic,rmodes,inertia,ieigv)

      call rkeywd('akira_control','$prnmod',prnmod)
      if(prnmod) then
        do i=1,3
          call printdmat(tmodes(1,i),3*natoms,1,'tmodes','f20.12',ftape)
        end do
        do i=1,3
          call printdmat(rmodes(1,i),3*natoms,1,'rmodes','f20.12',ftape)
        end do
      end if

*-------------------------------------------------------
* deallocate memory 
*-------------------------------------------------------
      deallocate(nco)

      end

************************************************************************
   
      subroutine move2cm(co,natoms,ic,nco)

*----------------------------------------------------------------------
* set up center-of-mass coordinate system
*
* written by J. Neugebauer, Uni Erlangen, 2002
*----------------------------------------------------------------------
      implicit none
      integer natoms,i,j
      integer ic(natoms)
      real*8 co(3,natoms),nco(3,natoms),com(3)
      
*******
* determine center of mass
*******
      call getcom(natoms,co,ic,com)

*******
* set up new (center-of-mass) coordinates
*******
      do i=1,natoms
        do j=1,3
          nco(j,i)=co(j,i)-com(j)
        end do
      end do

      end

************************************************************************
   
      subroutine pamitrans(coord,natoms,ic,inertia,ieigval)
*----------------------------------------------------------------------
* transformation into the basis of the eigenvectors of the moment of
* inertia matrix
*
* written by J. Neugebauer, Uni Erlangen, 2002
*----------------------------------------------------------------------
      implicit none
      integer natoms,i,j,k,info
      integer ic(natoms)
      real*8 ae,be,ce,coord(3,natoms),find_mass,com(3),kdelta,kd
      real*8 inertia(3,3),ieigval(3),worka(8),scri,tco(3)
      external find_mass,kdelta

*******
* determine matrix of moments of inertia:
*
*   I(i,j) = sum_k  m_k * [(delta_{i,j}*sum_p coord_p*coord_p) - coord_i*coord_j]
*
*******
      do i=1,3
        do j=1,3
          kd=kdelta(i,j)
          inertia(i,j)=0.0d0
          do k=1,natoms
            scri=kd*(coord(1,k)*coord(1,k))
            scri=scri+kd*(coord(2,k)*coord(2,k))
            scri=scri+kd*(coord(3,k)*coord(3,k))
            scri=scri-coord(i,k)*coord(j,k)
            inertia(i,j)=inertia(i,j)+scri*find_mass(ic(k))
          end do
        end do
      end do
 
*******
* solve secular determinant, calculate eigenvectors and eigenvalues
*******
      call dsyev('V','L',3,inertia,3,ieigval,worka,8,info)
      if(info.ne.0) call serror('pamitrans',2,
     * 'Could not calculate moments of inertia',6)

*******
* transform nuclear coordinates into eigenvector basis
* (multiply nuclear coordinates with transpose(inertia))
*******
      do i=1,natoms
        do j=1,3
          tco(j)=coord(j,i)
          coord(j,i)=0.0d0
        end do
        call matmul3t(coord(1,i),inertia,tco,1)
      end do
      end

************************************************************************

      subroutine gettrans(natoms,ic,tmodes)

*--------------------------------------------------------------------
* create pure translational modes
*
* written by J. Neugebauer, Uni Erlangen, 2002
*--------------------------------------------------------------------
      implicit none
      integer natoms,ic(natoms),i,j
      real*8 tmodes(3*natoms,3),find_mass
      external find_mass
      do j=1,3*natoms
        do i=1,3
          tmodes(j,i)=0.0d0
        end do
      end do
      do j=1,3
        do i=1,natoms
          tmodes((i-1)*3+j,j)=sqrt(find_mass(ic(i)))
        end do
      end do

*--------------
* normalization
*--------------
      do j=1,3
        call normalize(tmodes(1,j),3*natoms)
      end do

      end

************************************************************************

      subroutine getrot(coord,natoms,ic,rmodes,inertia,ieigv)

*--------------------------------------------------------------------
* create pure rotational modes
*
* written by J. Neugebauer, Uni Erlangen, 2002
*--------------------------------------------------------------------
      implicit none
      integer natoms,ic(natoms),i,j,icoo,isort(3)
      real*8 rmodes(3*natoms,3),find_mass,inertia(3,3),ieigv(3)
      real*8 scr(3),scr2(3),coord(3,natoms)
      external find_mass

      do i=1,natoms
        icoo=3*(i-1)
* Rx
        scr(1)=0.0d0
        scr(2)=-coord(3,i)
        scr(3)=coord(2,i)
        call matmul3(scr2,inertia,scr,1)
        do j=1,3
          rmodes(icoo+j,1)=sqrt(find_mass(ic(i)))*scr2(j)
        end do

* Ry
        scr(1)=coord(3,i)
        scr(2)=0.0d0
        scr(3)=-coord(1,i)
        call matmul3(scr2,inertia,scr,1)
        do j=1,3
          rmodes(icoo+j,2)=sqrt(find_mass(ic(i)))*scr2(j)
        end do

* Rz
        scr(1)=-coord(2,i)
        scr(2)=coord(1,i)
        scr(3)=0.0d0
        call matmul3(scr2,inertia,scr,1)
        do j=1,3
          rmodes(icoo+j,3)=sqrt(find_mass(ic(i)))*scr2(j)
        end do
      end do

*--------------
* normalization
*--------------
      do j=1,3
        call normalize(rmodes(1,j),3*natoms)
      end do
      end

***********************************************************************

      subroutine matmul3(amat,bmat,cmat,n)

* matrix multiplication:
* amat(3,n) = bmat(3,3)*cmat(3,n)
*
* written by J. Neugebauer, Uni Erlangen, 2002

      implicit none
      integer i,j,k,n
      real*8 amat(3,n),bmat(3,3),cmat(3,n),scr
      do i=1,n
        do j=1,3
          scr=0.0d0
          do k=1,3
            scr=scr+bmat(j,k)*cmat(k,i)
          end do
          amat(j,i)=scr
        end do 
      end do
      end 

***********************************************************************

      subroutine matmul3t(amat,bmat,cmat,n)

* matrix multiplication:
* amat(3,n) = transpose(bmat(3,3))*cmat(3,n)
*
* written by J. Neugebauer, Uni Erlangen, 2002

      implicit none
      integer i,j,k,n
      real*8 amat(3,n),bmat(3,3),cmat(3,n),scr
      do i=1,n
        do j=1,3
          scr=0.0d0
          do k=1,3
            scr=scr+bmat(k,j)*cmat(k,i)
          end do
          amat(j,i)=scr
        end do 
      end do
      end 
